/*
  TCP-Server und SuchTon-Ausgabe für Kabelsuche
  Change SSID and PASSWORD.
*/

#define SSID "Total_Commander"
#define PASSWORD "HE_T_Hdz"

#define LED_WLAN 13

#define SENSOR A0
#define LED 6
#define button 7
#define GEN_AUS 9
#define GEN_AU2 10
#define button2 A2
#define DEBUG true


#include <SoftwareSerial.h>
#include "Webseite.h"

SoftwareSerial esp8266(11, 12); // RX, TX


long readTemp()

{

  long result; // Read temperature sensor against 1.1V reference

  ADMUX = _BV(REFS1) | _BV(REFS0) | _BV(MUX3);

  delay(20); // Wait for Vref to settle - 2 was inadequate

  ADCSRA |= _BV(ADSC); // Convert

  while (bit_is_set(ADCSRA, ADSC));

  result = ADCL;

  result |= ADCH << 8;

  result = (result - 125) * 1075; //Don't forget to change "x" with multiplication sign

  return result;

}

long readVcc()

{

  long resultVcc; // Read 1.1V reference against AVcc

  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);

  delay(20); // Wait for Vref to settle - 2 was inadequate

  ADCSRA |= _BV(ADSC); // Convert

  while (bit_is_set(ADCSRA, ADSC));

  resultVcc = ADCL;

  resultVcc |= ADCH << 8;

  resultVcc = 1126400L / resultVcc; // Back-calculate AVcc in mV

  return resultVcc;

}


boolean schalter;


void setup() {
  pinMode(LED, OUTPUT);
  pinMode(GEN_AUS, OUTPUT);
  pinMode(GEN_AU2, OUTPUT);
  pinMode(button, INPUT_PULLUP);
  pinMode(SENSOR, INPUT);
  pinMode(button2, INPUT);
  Serial.begin(19200);
  esp8266.begin(19200);
  schalter = false;
  debug("Suche WLAN:"); debug(SSID);
  if (!espConfig()) serialDebug();
  else BlinkWLAN(LED_WLAN, 1, 10000);          //WLAN-LED für 10 sek an. Danach Stromsparen

  if (configTCPServer())  debug("Server Aktiv"); else debug("Server Error");
}

void loop() {
  String xBuffer; int hlf = 0;
  int Frequenz = analogRead(SENSOR);
  if (esp8266.available()) // check if the esp is sending a message
  {
    if (esp8266.find("+IPD,"))
    {
      debug("Incomming Request");
      int connectionId = esp8266.parseInt();
      if (esp8266.findUntil("&START_STOP=", "\n")) hlf = esp8266.parseInt();
      debug(String(hlf));
      schalter = hlf > 0;
      if (sendWebsite(connectionId, createWebsite())) {
        debug("Website send OK");
        BlinkWLAN(LED_WLAN, 3, 500);
      } else {
        debug("Website send Error");
        BlinkWLAN(LED_WLAN, 20, 100);
      }
    }

  }


  if (!digitalRead(button))
  {
    noTone(GEN_AUS);                                  //Button S1 alleine zeigt die verwendete SuchFrequenz als Helligkeit an
    analogWrite(LED, map(Frequenz, 0, 1023, 0, 255)); //dafür Muss aber der Ton AUS sein, da sich PWM und tone gegenseitig beeinflussen
  }
  else
  {
    digitalWrite(LED, LOW);
    if (schalter)
    {
      digitalWrite(GEN_AU2, LOW);
      tone(GEN_AUS, Frequenz + 31, 400);
      delay(100);
      tone(GEN_AUS, Frequenz * 2, 400);
      delay(100);

    }
  }
  if (analogRead(button2) > 1000)               //Zu Erst ButtonS2 drücken und kurz S1 dazu schaltet den Suchmodus An und Aus
  {
    if (!digitalRead(button))
    {
      while (!digitalRead(button)) {}
      schalter = !schalter;
    }
    else
    {
      //Nur Button S2 zeigt den Status an
      ZeigeStatus();
    }
  }
}

void ZeigeStatus()
{
  if (schalter)
  {
    BlinkWLAN(LED, 2, 1000);
    //Wenn der Sucher gestartet worden ist, dann Leuchte 2mal für 1 sek.

  }
  else BlinkWLAN(LED, 2, 100);  //an sonsten 2mal schneller
}

void BlinkWLAN(int LEDs, int zahl, int zeit)         // Kleiner debug-Ersatz (ohne delay)
{ long z, t;
  for (int i = 0; i < zahl; i++)
  {
    digitalWrite(LEDs, HIGH);
    z = millis(); t = z;
    while (t <= z + zeit)
    {
      t = millis();
    }
    digitalWrite(LEDs, LOW);
    z = millis();
    while (t <= z + zeit)
    {
      t = millis();
    }
  }
}
//---------------------------Webseite

boolean sendWebsite(int connectionId, String webpage)
{
  boolean succes = true;

  if (sendCom("AT+CIPSEND=" + String(connectionId) + "," + String(webpage.length()), ">"))
  {
    esp8266.print(webpage);
    esp8266.find("SEND OK");
    succes &= sendCom("AT+CIPCLOSE=" + String(connectionId), "OK");
  }
  else
  {
    succes = false;
  }
  return succes;
}

String createWebsite()
{
  String xBuffer;
  float tmp;
  float vcc;

  tmp = readTemp() / 10000.0;
  delay(20);
  vcc = readVcc() / 1000.0;
  for (int i = 0; i < sizeof(site); i++)
  {
    char myChar = pgm_read_byte_near(site + i);
    xBuffer += myChar;
  }

  xBuffer.replace("*bright*", String(analogRead(SENSOR)));
  xBuffer.replace("*Temp*", String(tmp));
  xBuffer.replace("*Volts*", String(vcc));
  if (schalter)
    xBuffer.replace("*AN_AUS*", "AN");
  else
    xBuffer.replace("*AN_AUS*", "AUS");
  return xBuffer;
}


//-----------------------------------------Config ESP8266------------------------------------

boolean espConfig()
{
  boolean succes = true;
  esp8266.setTimeout(5000);
  succes &= sendCom("AT+RST", "ready");
  esp8266.setTimeout(1000);
  if (configStation(SSID, PASSWORD)) {
    succes &= true;
    debug("WLAN Connected");
    debug("My IP is:");
    debug(sendCom("AT+CIFSR"));
  }
  else
  {
    if (configAP())
    {
      succes &= true;
      debug("WLAN AP aktiviert");
      debug("My IP is:");
      debug(sendCom("AT+CIFSR"));
    }
    else
      succes &= false;
  }
  //shorter Timeout for faster wrong UPD-Comands handling
  succes &= sendCom("AT+CIPMODE=0", "OK");
  succes &= sendCom("AT+CIPMUX=0", "OK");

  return succes;
}

boolean configTCPServer()
{
  boolean succes = true;

  succes &= (sendCom("AT+CIPMUX=1", "OK"));
  succes &= (sendCom("AT+CIPSERVER=1,80", "OK"));

  return succes;

}

boolean configTCPClient()
{
  boolean succes = true;

  succes &= (sendCom("AT+CIPMUX=0", "OK"));
  //succes &= (sendCom("AT+CIPSERVER=1,80", "OK"));

  return succes;

}


boolean configStation(String vSSID, String vPASSWORT)
{
  boolean succes = true;
  succes &= (sendCom("AT+CWMODE=1", "OK"));
  esp8266.setTimeout(20000);
  succes &= (sendCom("AT+CWJAP=\"" + String(vSSID) + "\",\"" + String(vPASSWORT) + "\"", "OK"));
  esp8266.setTimeout(1000);
  return succes;
}

boolean configAP()
{
  boolean succes = true;

  succes &= (sendCom("AT+CWMODE=2", "OK"));
  succes &= (sendCom("AT+CWSAP=\"NanoESP\",\"\",5,0", "OK"));

  return succes;
}


//-----------------------------------------------Controll ESP-----------------------------------------------------


boolean sendCom(String command, char respond[])
{
  esp8266.println(command);
  if (esp8266.findUntil(respond, "ERROR"))
  {
    return true;
  }
  else
  {
    debug("ESP SEND ERROR: " + command);
    return false;
  }
}

String sendCom(String command)
{
  esp8266.println(command);
  return esp8266.readString();
}



//-------------------------------------------------Debug Functions------------------------------------------------------
void serialDebug() {
  while (true)
  {
    if (esp8266.available())
      Serial.write(esp8266.read());
    if (Serial.available())
      esp8266.write(Serial.read());
  }
}

void debug(String Msg)
{
  if (DEBUG)
  {
    Serial.println(Msg);
  }
}
